#include "libs.h"
#include "rigid_body.h"
#include "space.h"
#include "objimport.h"
#include "frame.h"

RigidBody::RigidBody(void) : StaticRigidBody() {
  m_flags = Body::FLAG_CAN_MOVE_FRAME;
  m_body  = dBodyCreate(Space::world);
  dMassSetBox(&m_mass, 1, 50, 50, 50);
  dMassAdjust(&m_mass, 1.0f);

  dBodySetMass(m_body, &m_mass);
  dGeomSetBody(m_geom, m_body);
  SetPosition(vector3d(0,0,0));
}

vector3d RigidBody::GetAngularMomentum(void) {
  matrix4x4d I;
  I.LoadFromOdeMatrix(m_mass.I);
  return I * vector3d(dBodyGetAngularVel(m_body));
}

RigidBody::~RigidBody(void) {
  dBodyDestroy(m_body);
}

void RigidBody::SetVelocity(vector3d v) {
  dBodySetLinearVel(m_body, v.x, v.y, v.z);
}

void RigidBody::SetAngVelocity(vector3d v) {
  dBodySetAngularVel(m_body, v.x, v.y, v.z);
}

